/* DriverLib Includes */
#include <ti/devices/msp432e4/driverlib/driverlib.h>

/* Standard Includes */

/* Defines for I2C bus parameters */
#define SLAVE_ADDRESS   0x48
#define I2C_NUM_DATA    2
#define CONFIGURATION   0x01
#define TEMP_RESULT     0x00

/* Variables for I2C data and state machine */
volatile uint8_t  sendMasterTxData[I2C_NUM_DATA];
uint8_t           getMasterRxData[I2C_NUM_DATA] = {0};
const    uint8_t  setMasterBytesLength = I2C_NUM_DATA;

/* The control table used by the uDMA controller.  This table must be aligned
 * to a 1024 byte boundary. */
#if defined(__ICCARM__)
#pragma data_alignment=1024
uint8_t pui8ControlTable[1024];
#elif defined(__TI_ARM__)
#pragma DATA_ALIGN(pui8ControlTable, 1024)
uint8_t pui8ControlTable[1024];
#else
uint8_t pui8ControlTable[1024] __attribute__ ((aligned(1024)));
#endif

uint32_t getInterruptStatus;
bool I2Cready = false;

void I2C5_IRQHandler(void)
{
    getInterruptStatus = I2CMasterIntStatusEx(I2C5_BASE, true);
    I2CMasterIntClearEx(I2C5_BASE, getInterruptStatus);

    I2Cready = true;
}

void GPIOP3_IRQHandler(uint_least8_t index)
{
    GPIOIntDisable(GPIO_PORTP_BASE, GPIO_INT_PIN_3);  // disable TEMP_DRDY interrupt until we service this one
    GPIOIntClear(GPIO_PORTP_BASE, GPIO_INT_PIN_3);

    if (!I2CMasterBusy(I2C5_BASE)){
        I2CMasterSlaveAddrSet(I2C5_BASE, SLAVE_ADDRESS, true);
        I2CMasterControl(I2C5_BASE, I2C_MASTER_CMD_FIFO_SINGLE_RECEIVE);
    }

    GPIOIntEnable(GPIO_PORTP_BASE, GPIO_INT_PIN_3);   // reenable TEMP_DRDY interrupt
}

int send_config(void)
{
    I2CMasterSlaveAddrSet(I2C5_BASE, SLAVE_ADDRESS, false);
    I2CMasterDataPut(I2C5_BASE, CONFIGURATION);
    I2CMasterControl(I2C5_BASE, I2C_MASTER_CMD_BURST_SEND_START);

    while (!I2Cready);
    I2Cready = false;

    if ((getInterruptStatus & I2C_MASTER_INT_NACK))
        return -1;

    I2CMasterControl(I2C5_BASE, I2C_MASTER_CMD_FIFO_BURST_SEND_FINISH);

    while (getInterruptStatus & I2C_MASTER_INT_TX_DMA_DONE);

    while (!(getInterruptStatus & I2C_MASTER_INT_STOP));

    I2CMasterIntClearEx(I2C5_BASE, getInterruptStatus);
    I2Cready = false;

    I2CMasterSlaveAddrSet(I2C5_BASE, SLAVE_ADDRESS, false);
    I2CMasterDataPut(I2C5_BASE, TEMP_RESULT);
    I2CMasterControl(I2C5_BASE, I2C_MASTER_CMD_BURST_SEND_START);

    while (!I2Cready);
    I2Cready = false;

    if ((getInterruptStatus & I2C_MASTER_INT_NACK))
        return -1;

    I2CMasterSlaveAddrSet(I2C5_BASE, SLAVE_ADDRESS, true);
    I2CMasterControl(I2C5_BASE, I2C_MASTER_CMD_FIFO_SINGLE_RECEIVE);

    while (getInterruptStatus & I2C_MASTER_INT_RX_DMA_DONE);

    while (!(getInterruptStatus & I2C_MASTER_INT_STOP));

    return 0; 
}

void
ConfigureuDMATX(uint8_t ui8Length, uint32_t ui32ArbSize)
{
    uDMAChannelAttributeDisable(UDMA_CH23_I2C5TX, UDMA_ATTR_USEBURST |
                                    UDMA_ATTR_ALTSELECT |
                                    UDMA_ATTR_HIGH_PRIORITY |
                                    UDMA_ATTR_REQMASK);

    uDMAChannelControlSet(UDMA_CH23_I2C5TX | UDMA_PRI_SELECT,
                              UDMA_SIZE_8 | UDMA_SRC_INC_8 | UDMA_DST_INC_NONE |
                              ui32ArbSize);

    uDMAChannelTransferSet(UDMA_CH23_I2C5TX | UDMA_PRI_SELECT,
                               UDMA_MODE_BASIC, (void *)&sendMasterTxData,
                               (void *)&I2C5->FIFODATA, ui8Length);

    uDMAChannelEnable(UDMA_CH23_I2C5TX);
}

void
ConfigureuDMARX(uint8_t ui8Length, uint32_t ui32ArbSize)
{
    uDMAChannelAttributeDisable(UDMA_CH22_I2C5RX, UDMA_ATTR_USEBURST |
                                                     UDMA_ATTR_ALTSELECT |
                                                     UDMA_ATTR_HIGH_PRIORITY |
                                                     UDMA_ATTR_REQMASK);

    uDMAChannelControlSet(UDMA_CH22_I2C5RX | UDMA_PRI_SELECT,
                          UDMA_SIZE_8 | UDMA_SRC_INC_NONE | UDMA_DST_INC_8 |
                          ui32ArbSize);

    uDMAChannelTransferSet(UDMA_CH22_I2C5RX | UDMA_PRI_SELECT,
                           UDMA_MODE_BASIC, (void *)&I2C5->FIFODATA,
                           (void *)&getMasterRxData, ui8Length);

    uDMAChannelEnable(UDMA_CH22_I2C5RX);
}

int main(void)
{
    uint32_t systemClock;
    uint32_t setTxArbSize, setRxArbSize;

    /* Configure the system clock for 120 MHz */
    systemClock = SysCtlClockFreqSet((SYSCTL_XTAL_25MHZ | SYSCTL_OSC_MAIN |
                                          SYSCTL_USE_PLL | SYSCTL_CFG_VCO_480),
//                                          120000000);
                                          25000000);

    /* Enable the DMA and Configure Channel for I2C1 TX and RX for Basic mode
     * of transfer */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UDMA);
    while(!(SysCtlPeripheralReady(SYSCTL_PERIPH_UDMA)))
    {
    }

    uDMAEnable();

    /* Point at the control table to use for channel control structures. */
    uDMAControlBaseSet(pui8ControlTable);

    /* Assign the UDMA Channel for I2C1 RX and TX DMA Request */
    uDMAChannelAssign(UDMA_CH22_I2C5RX);
    uDMAChannelAssign(UDMA_CH23_I2C5TX);

    /* Enable clocks to GPIO Port G and configure pins as I2C */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    while(!(SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOB)))
    {
    }

    GPIOPinConfigure(GPIO_PB0_I2C5SCL);
    GPIOPinConfigure(GPIO_PB1_I2C5SDA);
    GPIOPinTypeI2C(GPIO_PORTB_BASE, GPIO_PIN_1);
    GPIOPinTypeI2CSCL(GPIO_PORTB_BASE, GPIO_PIN_0);

    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOP);
    while(!(SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOP)))
    {
    }
    GPIOPinTypeGPIOInput(GPIO_PORTP_BASE, GPIO_PIN_3);

    GPIOIntTypeSet(GPIO_PORTP_BASE, GPIO_PIN_3, (GPIO_FALLING_EDGE | GPIO_DISCRETE_INT));
    GPIOIntEnable(GPIO_PORTP_BASE, GPIO_INT_PIN_3);

    /* Enable the clock to I2C-1 module and configure the I2C Master */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_I2C5);
    while(!(SysCtlPeripheralReady(SYSCTL_PERIPH_I2C5)))
    {
    }

    /* Configure the I2C Master in standard mode and enable interrupt for
     * Arbitration Lost, Stop, NAK, Timeout and Data completion condition
     * on the bus */
    I2CMasterInitExpClk(I2C5_BASE, systemClock, true);
    I2CMasterIntEnableEx(I2C5_BASE, (I2C_MASTER_INT_ARB_LOST |
                                         I2C_MASTER_INT_STOP |
                                         I2C_MASTER_INT_NACK |
                                         I2C_MASTER_INT_TIMEOUT |
                                         I2C_MASTER_INT_DATA));

    /* Assign the Transmit and Receive FIFO to the Master Transmit threshold
     * of 2 means that when there are less than or equal to 2 bytes in the TX
     * FIFO then generate an interrupt.
     * Receive threshold of 6 means that when there are more than or equal to
     * 6 bytes in the RX FIFO then generate an interrupt. */
    I2CTxFIFOConfigSet(I2C5_BASE, I2C_FIFO_CFG_TX_MASTER_DMA |
                                      I2C_FIFO_CFG_TX_TRIG_2);
    I2CRxFIFOConfigSet(I2C5_BASE, I2C_FIFO_CFG_RX_MASTER_DMA |
                                      I2C_FIFO_CFG_RX_TRIG_3);

    /* Transmit uDMA Arbitrartion size is calculated as the
     * smaller mod 2 which can write to the TXFIFO without
     * overrun*/
    setTxArbSize = UDMA_ARB_4;

    /* Flush any existing data in the FIFO */
    I2CTxFIFOFlush(I2C5_BASE);
    I2CRxFIFOFlush(I2C5_BASE);

    /* Enable the interrupt generation from I2C-1 */
    IntEnable(INT_I2C5);

    /* Initialize the Master Transmit Buffer with Random Data and clear the
     * Master Receive Buffer */
    sendMasterTxData[0] = 0x01;
    sendMasterTxData[1] = 0xA4;

    /* Initialize the state of the I2C Master */
    I2CMasterBurstLengthSet(I2C5_BASE, setMasterBytesLength);

    I2CMasterIntEnableEx(I2C5_BASE, I2C_MASTER_INT_TX_DMA_DONE);
    I2CMasterIntEnableEx(I2C5_BASE, I2C_MASTER_INT_RX_DMA_DONE);

    /* Configure the uDMA Control Channel Structure for TX */
    ConfigureuDMATX(setMasterBytesLength, setTxArbSize);

    /* Receive uDMA Arbitrartion size is calculated as the
     * Trigger Level to read back data from RXFIFO without
     * an underrun */
    setRxArbSize = UDMA_ARB_2;

    /* Configure the uDMA Control Channel Structure for RX */
    ConfigureuDMARX(setMasterBytesLength,setRxArbSize);

    /* Set the I2CMBLEN register and also initialize */
    I2CMasterBurstLengthSet(I2C5_BASE, setMasterBytesLength);

    send_config();

    I2CMasterIntEnableEx(I2C5_BASE, I2C_MASTER_INT_RX_DMA_DONE);
    I2CMasterIntDisableEx(I2C5_BASE, I2C_MASTER_INT_TX_DMA_DONE);

    IntEnable(INT_GPIOP3);

    while(1);
}
